@@ -4,6 +4,7 @@
 #include <unistd.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
+#include <libosso.h>
 #include "level.h"
 #include "images.h"
 #include "socket.h"
@@ -14,6 +15,7 @@ HgwContext *hgw_context = NULL;
 #endif
 #include "callbacks.h"
 
+osso_context_t * osso_context = NULL;
 GAMEMODE gm;
 SDL_mutex *mutex;
 GameState game_state;
@@ -25,6 +27,8 @@ int game_loop() {
     SDL_Event event;
     unsigned long t1, t2;
     int no_unselect = 0;
+    int has_focus = 1;
+    Uint32 timeout = 5000;
 
     // Main loop
     t1 = SDL_GetTicks();
@@ -50,9 +54,11 @@ int game_loop() {
             dest.x = 386;
             dest.y = 151;
             dest.w = dest.h = 0;
-            SDL_BlitSurface(game_state.state == GAME_WIN ? win : lose, NULL, screen, &dest);
-            SDL_Update(386, 151, 365, 178);
-            SDL_Delay(4000);
+            if (game_state.state != GAME_EXIT) {
+                SDL_BlitSurface(game_state.state == GAME_WIN ? win : lose, NULL, screen, &dest);
+                SDL_Update(386, 151, 365, 178);
+                SDL_Delay(4000);
+            }
             return (game_state.state == GAME_LOSE);
         }
         
@@ -63,6 +69,7 @@ int game_loop() {
     
                 // Quit
                 if (event.key.keysym.sym == SDLK_ESCAPE
+                            || event.key.keysym.sym == SDLK_BACKSPACE
                             || event.key.keysym.sym == SDLK_F4
                             || event.key.keysym.sym == SDLK_F5 
                             || event.key.keysym.sym == SDLK_F6) {
@@ -76,6 +83,13 @@ int game_loop() {
                 case SDL_QUIT:
 		    game_state.state = GAME_EXIT;
                     break;
+
+		// Window focus change
+		case SDL_ACTIVEEVENT:
+		    if (event.active.gain == 0 && gm != GM_MULTIPLAYER) {
+		        game_state.state = GAME_EXIT;
+		    }
+		    break;
                     
                 // Button pressed
                 case SDL_MOUSEBUTTONUP:
@@ -184,6 +198,14 @@ int game_loop() {
 			draw_timer_bar(true);
 		}
 	}
+
+	// Timeout
+	if (has_focus || gm == GM_MULTIPLAYER) timeout = SDL_GetTicks() + 5000;
+	if (SDL_GetTicks() > timeout) {
+		game_state.state = GAME_EXIT;
+	}
+
+
     }
 }
 
@@ -220,8 +242,9 @@ void show_menu_loop() {
 		quit_callback(0);
 	} else exit_callback(0);
     } else {
+	    int joined;
 	    if (createserver) create_game();
-	    else join_game(ip);
+	    else joined = join_game(ip);
 
             while (1) {
 
@@ -282,6 +305,7 @@ int main(int argc, char **argv) {
 		return 0;
 	}
 	SDL_ShowCursor(SDL_DISABLE);
+	SDL_WM_SetCaption("Battle Gweled", NULL);
 
 	// Load images
 	if (!load_surfaces()) {
@@ -298,7 +322,11 @@ int main(int argc, char **argv) {
     SDL_DestroyMutex(mutex);
 
 #if HGW_FUNC
-	hgw_context_compat_destroy_deinit(hgw_context);
+        if (game_state.state == GAME_LOSE)
+		hgw_context_destroy(hgw_context, HGW_BYE_INACTIVE);
+	else
+		hgw_context_destroy(hgw_context, HGW_BYE_PAUSED);
+
 #endif
 
 	// Finalize game
